window.Gleam.initSearch([{"doc":"stratus","title":"stratus","content":"# stratus\n\nA Gleam WebSocket client (and possibly more).\n\n[![Package Version](https://img.shields.io/hexpm/v/stratus)](https://hex.pm/packages/stratus)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/stratus/)\n\n```sh\ngleam add stratus\n```\n\n## Features\n- [x] Sending and receiving Text and Binary frames\n- [x] Custom user messages\n- [ ] Sending and receiving continuation frames\n- [x] Per-message deflate\n- [ ] Graceful shutdown\n\n## Example\n\n```gleam\nimport birl\nimport gleam/erlang/process\nimport gleam/function\nimport gleam/http/request\nimport gleam/io\nimport gleam/option.{None}\nimport gleam/otp/actor\nimport logging\nimport repeatedly\nimport stratus\n\npub type Msg {\n  Close\n  TimeUpdated(String)\n}\n\npub type LogLevel {\n  Debug\n}\n\npub type Log {\n  Level\n}\n\n@external(erlang, &quot;logger&quot;, &quot;set_primary_config&quot;)\nfn set_logger_level(log: Log, level: LogLevel) -&gt; Nil\n\npub fn main() {\n  logging.configure()\n  // NOTE:  This isn&#39;t strictly necessary at all (including the associated\n  // stuff above). It&#39;s included just to show the debug logging.\n  set_logger_level(Level, Debug)\n  let assert Ok(req) = request.to(&quot;http://localhost:3000/ws&quot;)\n  let builder =\n    stratus.websocket(\n      request: req,\n      init: fn() { #(Nil, None) },\n      loop: fn(msg, state, conn) {\n        case msg {\n          stratus.Text(_msg) -&gt; {\n            let assert Ok(_resp) =\n              stratus.send_text_message(conn, &quot;hello, world!&quot;)\n            actor.continue(state)\n          }\n          stratus.User(TimeUpdated(msg)) -&gt; {\n            let assert Ok(_resp) = stratus.send_text_message(conn, msg)\n            actor.continue(state)\n          }\n          stratus.Binary(_msg) -&gt; actor.continue(state)\n          stratus.User(Close) -&gt; {\n            let assert Ok(_) = stratus.close(conn)\n            actor.Stop(process.Normal)\n          }\n        }\n      },\n    )\n    |&gt; stratus.on_close(fn(_state) { io.println(&quot;oh noooo&quot;) })\n\n  let assert Ok(subj) = stratus.initialize(builder)\n\n  let timer =\n    repeatedly.call(1000, Nil, fn(_state, _count_) {\n      let now =\n        birl.now()\n        |&gt; birl.to_iso8601\n      stratus.send_message(subj, TimeUpdated(now))\n    })\n\n  process.start(\n    fn() {\n      process.sleep(6000)\n\n      stratus.send_message(subj, Close)\n    },\n    True,\n  )\n\n  let done =\n    process.new_selector()\n    |&gt; process.selecting_process_down(\n      process.monitor_process(process.subject_owner(subj)),\n      function.identity,\n    )\n    |&gt; process.select_forever\n\n  io.debug(#(&quot;WebSocket process exited&quot;, done))\n\n  repeatedly.stop(timer)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/stratus&gt;.\n","url":"index.html"},{"doc":"stratus","title":"Builder","content":"pub opaque type Builder(state, user_message)\n\n\nSynonyms:\nstratus.Builder\nstratus Builder","url":"stratus.html#Builder"},{"doc":"stratus","title":"Connection","content":"pub opaque type Connection\n This holds some information needed to communicate with the WebSocket.\n\nSynonyms:\nstratus.Connection\nstratus Connection","url":"stratus.html#Connection"},{"doc":"stratus","title":"InternalMessage","content":"pub opaque type InternalMessage(user_message)\n These are the messages emitted or received by the underlying process.  You\n should only need to interact with `Message` below.\n\nSynonyms:\nstratus.InternalMessage\nstratus InternalMessage","url":"stratus.html#InternalMessage"},{"doc":"stratus","title":"Message","content":"pub type Message(user_message) {\n  Text(String)\n  Binary(BitArray)\n  User(user_message)\n}\n This is the type of message your handler might receive.\nText(String)\n\n\nBinary(BitArray)\n\n\nUser(user_message)\n\n\nSynonyms:\nstratus.Message\nstratus Message","url":"stratus.html#Message"},{"doc":"stratus","title":"close","content":"pub fn close(conn: Connection) -&gt; Result(Nil, SocketReason)\n This will close the WebSocket connection.\nSynonyms:\nstratus.close\nstratus close","url":"stratus.html#close"},{"doc":"stratus","title":"initialize","content":"pub fn initialize(\n  builder: Builder(a, b),\n) -&gt; Result(Subject(InternalMessage(b)), StartError)\n This opens the WebSocket connection with the provided `Builder`. It makes\n some assumptions about the request if you do not provide it.  It will use\n ports 80 or 443 for `ws` or `wss` respectively.\n\n It will open the connection and perform the WebSocket handshake. If this\n fails, the actor will fail to start with the given reason as a string value.\n\n After that, received messages will be passed to your loop, and you can use\n the helper functions to send messages to the server. The `close` method will\n send a close frame and end the connection.\nSynonyms:\nstratus.initialize\nstratus initialize","url":"stratus.html#initialize"},{"doc":"stratus","title":"on_close","content":"pub fn on_close(\n  builder: Builder(a, b),\n  on_close: fn(a) -&gt; Nil,\n) -&gt; Builder(a, b)\n You can provide a function to be called when the connection is closed. This\n function receives the last value for the state of the WebSocket.\n\n NOTE:  If you manually call `stratus.close`, this function will not be\n called. I&#39;m unsure right now if this is a bug or working as intended. But\n you will be in the loop with the state value handy.\nSynonyms:\nstratus.on_close\nstratus on_close","url":"stratus.html#on_close"},{"doc":"stratus","title":"on_handshake_error","content":"pub fn on_handshake_error(\n  builder: Builder(a, b),\n  on_handshake_error: fn(Response(BitArray)) -&gt; Nil,\n) -&gt; Builder(a, b)\n If the WebSocket handshake fails, this method will be called with the\n response received from the server. The process will stop after this.\nSynonyms:\nstratus.on_handshake_error\nstratus on_handshake_error","url":"stratus.html#on_handshake_error"},{"doc":"stratus","title":"send_binary_message","content":"pub fn send_binary_message(\n  conn: Connection,\n  msg: BitArray,\n) -&gt; Result(Nil, SocketReason)\n From within the actor loop, this is how you send a WebSocket text frame.\nSynonyms:\nstratus.send_binary_message\nstratus send_binary_message","url":"stratus.html#send_binary_message"},{"doc":"stratus","title":"send_message","content":"pub fn send_message(\n  subject: Subject(InternalMessage(a)),\n  message: a,\n) -&gt; Nil\n Since the actor receives the raw data from the WebSocket, it needs a less\n ergonomic message type. You probably don&#39;t want (read: shouldn&#39;t be able to)\n send `Data(bits)` to the process, so that message type is opaque.\n\n To get around that, this helper method lets you provide your custom message\n type to the actor.\n\n This is likely what you want if you want to be able to tell the actor to\n send data to the server. Your message type would be -- in plain language --\n &quot;this thing happened&quot;, and your loop would then send whatever relevant data\n corresponds to that event.\nSynonyms:\nstratus.send_message\nstratus send_message","url":"stratus.html#send_message"},{"doc":"stratus","title":"send_ping","content":"pub fn send_ping(\n  conn: Connection,\n  data: BitArray,\n) -&gt; Result(Nil, SocketReason)\n Send a ping frame with some data.\nSynonyms:\nstratus.send_ping\nstratus send_ping","url":"stratus.html#send_ping"},{"doc":"stratus","title":"send_text_message","content":"pub fn send_text_message(\n  conn: Connection,\n  msg: String,\n) -&gt; Result(Nil, SocketReason)\n From within the actor loop, this is how you send a WebSocket text frame.\n This must be valid UTF-8, so it is a `String`.\nSynonyms:\nstratus.send_text_message\nstratus send_text_message","url":"stratus.html#send_text_message"},{"doc":"stratus","title":"websocket","content":"pub fn websocket(\n  request req: Request(String),\n  init init: fn() -&gt; #(a, Option(Selector(b))),\n  loop loop: fn(Message(b), a, Connection) -&gt; Next(b, a),\n) -&gt; Builder(a, b)\n This creates a builder to set up a WebSocket actor. This will use default\n values for the connection initialization timeout, and provide an empty\n function to be called when the server closes the connection. If you want to\n customize either of those, see the helper functions `with_connect_timeout`\nSynonyms:\nstratus.websocket\nstratus websocket","url":"stratus.html#websocket"},{"doc":"stratus","title":"with_connect_timeout","content":"pub fn with_connect_timeout(\n  builder: Builder(a, b),\n  timeout: Int,\n) -&gt; Builder(a, b)\n This sets the maximum amount of time you are willing to wait for both\n connecting to the server and receiving the upgrade response.  This means\n that it may take up to `timeout * 2` to begin sending or receiving messages.\n This value defaults to 5 seconds.\nSynonyms:\nstratus.with_connect_timeout\nstratus with_connect_timeout","url":"stratus.html#with_connect_timeout"},{"doc":"stratus","title":"stratus","content":"","url":"stratus.html"}]);